@@ -4,10 +4,10 @@ This file contains the Bond class, which is TmVal's class for representing bonds
 import numpy as np
 
 from math import floor
-from typing import Iterable, Union
+from typing import Iterable, List, Union
 
 from tmval.annuity import Annuity
-from tmval.growth import standardize_acc, TieredTime
+from tmval.growth import Amount, standardize_acc, TieredTime
 from tmval.rate import Rate
 from tmval.value import Payments
 
@@ -104,12 +104,25 @@ class Bond(Payments):
             self.fr = fr
             self.cfreq = cfreq
 
+        if [red, k].count(None) == 2:
+            red_args = None
+        else:
+            red_args = len([red, k])
+
         if term is not None:
             self.is_term_floor = self.term_floor()
         else:
             pass
 
-        args = [gr, red, price, term, c_args]
+        if [alpha, cfreq, fr, cgr].count(None) == 4:
+            self.is_zero = True
+        else:
+            self.is_zero = False
+
+        if self.is_zero:
+            args = [gr, red_args, price, term]
+        else:
+            args = [gr, red_args, price, term, c_args]
 
         n_missing = args.count(None)
 
@@ -126,7 +139,9 @@ class Bond(Payments):
                 self.red = red
                 self.price = price
 
-                if self.fr_is_level:
+                if self.is_zero:
+                    amounts = [-price, red]
+                elif self.fr_is_level:
                     amounts = [-price] + [self.fr] * self.n_coupons + [red]
                 else:
                     amounts = [-price]
@@ -150,6 +165,13 @@ class Bond(Payments):
 
                 self.coupons = self.get_coupons()
 
+            elif term is None:
+                if self.is_zero:
+                    self.red = red
+                    self.gr = standardize_acc(gr)
+                    self.price = price
+                    self.term = Amount(gr=gr, k=price).solve_t(fv=red)
+
             else:
                 self.n_coupons = self.get_n_coupons()
                 self.red = red
@@ -159,7 +181,7 @@ class Bond(Payments):
                 self.red = self.get_redemption()
 
         elif n_missing == 2:
-            if price is None and red is None:
+            if price is None and red_args is None:
                 if pd is not None:
                     self.n_coupons = self.get_n_coupons()
                     self.gr = standardize_acc(gr)
@@ -180,6 +202,7 @@ class Bond(Payments):
 
                     self.price = self.makeham()
                     self.term = self.gr.solve_t(pv=k, fv=self.red)
+                    self.n_coupons = self.get_n_coupons()
                     self.coupons = self.get_coupons()
                     self.is_term_floor = self.term_floor()
                 else:
@@ -204,8 +227,12 @@ class Bond(Payments):
         else:
             raise Exception("Unable to evaluate bond. Too many missing arguments.")
 
-        amounts = self.coupons.amounts + [self.red]
-        times = self.coupons.times + [self.term]
+        if self.is_zero:
+            amounts = [self.red]
+            times = [self.term]
+        else:
+            amounts = self.coupons.amounts + [self.red]
+            times = self.coupons.times + [self.term]
 
         Payments.__init__(
             self,
@@ -231,7 +258,9 @@ class Bond(Payments):
 
         self.append(amounts=[-self.price], times=[0])
 
-        if self.fr_is_level:
+        if self.is_zero:
+            pass
+        elif self.fr_is_level:
             self.j = self.gr.val(1 / self.cfreq) - 1
             self.base = self.fr / self.j
             self.g = self.fr / self.red
@@ -249,7 +278,9 @@ class Bond(Payments):
         :rtype: list
         """
 
-        if self.fr_is_level:
+        if self.is_zero:
+            times = []
+        elif self.fr_is_level:
             times = [(x + 1) * 1 / self.cfreq for x in range(self.n_coupons)]
         else:
             times = []
@@ -290,7 +321,11 @@ class Bond(Payments):
         :return: The redemption amount.
         :rtype: float
         """
-        c = (self.price - self.coupons.pv()) * self.gr.val(self.term)
+
+        if self.is_zero:
+            c = self.price * self.gr.val(self.term)
+        else:
+            c = (self.price - self.coupons.pv()) * self.gr.val(self.term)
         return c
 
     def get_n_coupons(
@@ -303,7 +338,10 @@ class Bond(Payments):
         :return: The number of coupons.
         :rtype: int
         """
-        if self.fr_is_level:
+
+        if self.is_zero:
+            n_coupons = 0
+        elif self.fr_is_level:
             # if term is evenly divisible by period, assume bond purchased at beginning of period
             if round(self.term % (1 / self.cfreq), 5) == 0:
                 n_coupons = self.term * self.cfreq
@@ -325,7 +363,9 @@ class Bond(Payments):
         :return: The bond coupons.
         :rtype: Annuity
         """
-        if self.fr_is_level:
+        if self.is_zero:
+            coupons = None
+        elif self.fr_is_level:
 
             if round(self.term % (1 / self.cfreq), 5) == 0:
 
@@ -398,8 +438,9 @@ class Bond(Payments):
         :rtype: float
         """
         k = self.k
-        j = self.gr.val(1/self.cfreq)
-        p = self.g / j * (self.red - k + k)
+        j = self.gr.val(1/self.cfreq) - 1
+
+        p = self.g / j * (self.red - k) + k
         return p
 
     def base_amount(self) -> float:
@@ -1096,7 +1137,10 @@ class Bond(Payments):
         :return: Whether the term provided coincides with bond issuance or a coupon payment.
         :rtype: bool
         """
-        if isinstance(self.alpha, (int, float)) or self.alpha is None:
+
+        if self.alpha is None and self.cfreq is None and self.fr is None:
+            return True
+        elif isinstance(self.alpha, (int, float)) or self.alpha is None and self.cfreq is not None:
             if round(self.term % (1 / self.cfreq), 5) == 0:
                 return True
             else:
@@ -1216,3 +1260,90 @@ def parse_cgr(
 #     j = cgr.rate / cgr.freq
 #
 #     acc = Accumulation(gr=j)
+
+
+def spot_rates(bonds: List[Bond] = None, yields=None, alpha=None):
+    """
+    Solves for the spot rates given a list of bonds.
+    :param bonds:
+    :type bonds:
+    :param yields:
+    :type yields:
+    :param alpha:
+    :type alpha:
+    :return:
+    :rtype:
+    """
+
+    if yields is not None and alpha is not None and bonds is None:
+        bonds = [Bond(
+            face=100,
+            red=100,
+            alpha=alpha,
+            cfreq=1,
+            term=t + 1,
+            gr=y
+        ) for t, y in zip(range(len(yields)), yields)]
+
+    # find bond with shortest term
+    bond_ts = [b.term for b in bonds]
+    min_t = min(bond_ts)
+    b0_i = bond_ts.index(min_t)
+    b0 = bonds[b0_i]
+
+    r_1 = b0.gr.interest_rate
+    rates = [r_1]
+
+    bonds.pop(0)
+
+    t0 = 1
+
+    res = {
+        t0: r_1
+    }
+
+    for b in bonds:
+        pmts = b.group_payments()
+        del pmts[0]
+        times = list(pmts.keys())
+        times.sort()
+        t_max = times.pop()
+        base = b.price
+        for r, t in zip(rates, times):
+            pv = pmts[t] / ((1 + r) ** t)
+            base -= pv
+        r_t = (pmts[t_max] / base) ** (1 / t_max) - 1
+        r_t = Rate(r_t)
+        rates += [r_t]
+        res.update({t_max: r_t})
+
+    return res
+
+
+def forward_rates(term, bonds=None, yields=None, alpha=None):
+    """
+    Given a list of bonds, or yields and coupon rates, returns the forward rates.
+
+    :param term:
+    :type term:
+    :param bonds:
+    :type bonds:
+    :param yields:
+    :type yields:
+    :param alpha:
+    :type alpha:
+    :return:
+    :rtype:
+    """
+    sr = spot_rates(bonds=bonds, yields=yields, alpha=alpha)
+
+    res = {}
+    for t in range(len(sr) + 1 - term):
+        if t == 0:
+            f = sr[term]
+        else:
+            f_factor = (1 + sr[t + term]) ** (t + term) / (1 + sr[t]) ** t
+            f = Rate(f_factor ** (1 / term) - 1)
+        res.update({(t, t + term): f})
+
+    return res
