@@ -12,6 +12,7 @@ from typing import (
     Callable,
     Iterable,
     Iterator,
+    List,
     Union
 )
 
@@ -78,6 +79,9 @@ class Payments:
         if gr is not None:
             self.set_accumulation(gr=gr)
 
+    def __add__(self, other):
+        self.append(amounts=other.amounts, times=other.times)
+
     def set_accumulation(self, gr: Union[float, Rate, Accumulation, TieredBal, TieredTime]):
 
         # if float, assume compound annual effective
@@ -664,3 +668,21 @@ def pairwise(iterable: Iterable) -> Iterator:
     a, b = itertools.tee(iterable)
     next(b, None)
     return zip(a, b)
+
+
+def extract_flows(payments: Union[Payments, List[Payments]]) -> Payments:
+
+    if isinstance(payments, Payments):
+        payments = [payments]
+    amounts = []
+    times = []
+    for pmts in payments:
+        amounts += pmts.amounts
+        times += pmts.times
+
+    res = Payments(
+        amounts=amounts,
+        times=times
+    )
+
+    return res
\ No newline at end of file
